#!/bin/bash
#

# build_h5py.sh 

usage()
{
   echo "USAGE: `basename ${0}` <product_directory> <e9|e10>:<p271[01]>[:mpich] <debug|prof> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

die()
{
    local exitval
    if [[ "$1" =~ ^[0-9]*$ ]]; then (( exitval = $1 )); shift; else (( exitval = 1 )); fi
    echo "ERROR: $@" 1>&2
    exit $exitval
}

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
    if [ -z ${SSIBUILDSHIMS_DIR} ]
    then
       echo "ERROR: failed to setup ssibuildshims ${ssibuildshims_version}"
       exit 1
    fi
}

set_qualifiers()
{
    if [ -z ${basequal} ]
    then
       echo "ERROR: please specify the qualifiers"
       usage
       exit 1
    fi

    qlist=(`echo ${basequal} | sed -e 's/:/ /g' | tr " " "\n"| sort |tr "\n" " "`)
    for qual in "${qlist[@]}"; do
        case $qual in
            e9|e10)
                cqual=$qual
                ;;
            p271[01])
                pqual=$qual
                ;;
            mpich)
                bqual=$qual
                export CC="${MPICC:-mpicc}"
                ;;
            *)
                die "Unrecognized qualifier $qual"
        esac
    done
    if [[ -z "$cqual" ]]; then
        die "Base qualifier is mandatory"
    fi
    if [[ -z "$pqual" ]] && (( "${cqual#e}" > 8 )); then
        die "A python qualifier is mandatory for e >= 9 (e.g. p2710)."
    fi
    case ${extraqual} in
        prof|debug)
            :
            ;;
        opt)
            die "${extraqual} builds of numpy are not supported at this time."
            ;;
        *)
            die "Unrecognized build qualifer ${extraqual}"
    esac
    fullqual=${cqual}${pqual:+:${pqual}}${bqual:+:${bqual}}:${extraqual}
    qualdir=`echo ${fullqual} | sed -e 's/:/-/g'`
}

get_platform_name()
{
    OS1=`uname`
    flvr=`ups flavor`
    if [ ${OS1} = "Darwin" ]
    then
	flvr=`ups flavor -2`
    fi
    OS=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform`
    plat=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform osplat`

    if [ -z ${flvr} ]
    then
       echo "ERROR: could not determine flavor for this platform"
       echo "       something is very wrong"
       exit 1
    fi

    pkgdir=${product_dir}/${package}/${pkgver}/${flvr}-${qualdir}
    patchdir=${product_dir}/${package}/${pkgver}/patch
    blddir=${product_dir}/${package}/${pkgver}/build/${flvr}
    tardir=${product_dir}/${package}/${pkgver}/tar
    sourcedir=${product_dir}/${package}/${pkgver}/source
}

set_compiler_flags()
{
    # We don't compile differently if debug, just use different dependencies.
    cflags="-O3 -g -DNDEBUG -fno-omit-frame-pointer"
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
extraqual=${3}
maketar=${4}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=h5py
origpkgver=v2_6_0
pkgver=${origpkgver}b
ssibuildshims_version=v0_10_01
setuptools_version=v22_0_5
nose_version=v1_3_7b
pkgdotver=`echo ${origpkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
pkgtarname=${package}-${pkgdotver}

get_this_dir
get_ssibuildshims
set_qualifiers
set_compiler_flags
get_platform_name

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
   exit 0
fi

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

mkdir -p ${blddir} || die "failed to create ${blddir}"

# declare now so we can setup
# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${fullqual} || \
  die "Fake declare of $package failed."

setup -B ${package} ${pkgver} -q ${fullqual} -z ${fakedb}:${product_dir}:${PRODUCTS} || die "fake setup failed"

setup -B setuptools ${setuptools_version} -q ${pqual} -z ${product_dir}:${PRODUCTS} || \
  die "unable to set up UPS package for python setuptools ${setuptools_version}"

ups active

if [ ${OS1} = "Darwin" ]
then
   PATH=`dropit /usr/bin/get-cert $PATH`
fi

set -x

python_shortdotversion=${PYTHON_VERSION#v}
python_shortdotversion=${python_shortdotversion//_/.}
python_shortdotversion=${python_shortdotversion%.*}

# work in a build directory, which will not be part of any distribution
mkdir -p ${pkgdir}/lib/python${python_shortdotversion}/site-packages || die "failed to create ${pkgdir}/lib/python2.7/site-packages"
cd ${blddir} || die "Unable to cd into ${blddir}"
tar xf ${tardir}/${pkgtarname}.tar.gz || die "Failed to untar ${pkgtarname}.tar.gz"
cd ${package}-${pkgdotver} || die "Failed to cd into ${package}-${pkgdotver}"

hdf5_dotversion=${HDF5_VERSION/_patch*/}
hdf5_dotversion=${hdf5_dotversion//_/.}
hdf5_dotversion=${hdf5_dotversion#v}

# Need HDF5_VERSION to be dotversion for python configure to work.
FC=false CXX=false HDF5_VERSION=${hdf5_dotversion} \
    python setup.py configure  --hdf5=${HDF5} ${bqual:+--mpi} || die "python setup.py configure failed"

FC=false CXX=false HDF5_VERSION=${hdf5_dotversion} \
    python setup.py install --prefix ${pkgdir} || die "python setup.py install failed"

# run some tests
#
setup -B nose $nose_version -q +${pqual} -z ${fakedb}:${product_dir}:${PRODUCTS} || \
  die "unable to set up python nose ${nose_version} -q +${pqual} for tests."
if (( ${UID} == 0 )) || (( ${EUID} == 0 )); then
  # A test fails under docker with AssertionError: 'r+' != 'r' so
  # disable for now: known issue, see
  # https://github.com/h5py/h5py/issues/696.
  :
else
  nosetests -w ${pkgdir}/lib/python${python_shortdotversion}/site-packages/h5py*/h5py/tests || die "nosetests failed"
fi

set +x

if [ ! -d ${pkgdir}/lib ]
then
   echo "ERROR: failed to create ${pkgdir}/lib"
   echo "       Something is very wrong"
   exit 1
fi

# real ups declare
${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${fullqual} || \
  die "failed to declare ${package} ${pkgver} -q ${fullqual}"

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup -B ${package} ${pkgver} -q ${fullqual} -z ${product_dir}:${PRODUCTS}
echo "h5py is installed at ${H5PY_FQ_DIR}"

# this must be last
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
fi

exit 0
